@@ -1,7 +1,7 @@
 /*	
  *   xtel - Emulateur MINITEL sous X11
  *
- *   Copyright (C) 1991-1994  Lectra Systemes & Pierre Ficheux
+ *   Copyright (C) 1991-1996  Lectra Systemes & Pierre Ficheux
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *   along with this program; if not, write to the Free Software
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-static char rcsid[] = "$Id: xteld.c,v 1.9 1995/07/24 13:42:41 pierre Exp $";
+static char rcsid[] = "$Id: xteld.c,v 1.20 1996/10/31 13:03:52 pierre Exp $";
 
 /*
  * Demon XTELD (communication avec le MODEM)
@@ -35,11 +35,16 @@ static char rcsid[] = "$Id: xteld.c,v 1.9 1995/07/24 13:42:41 pierre Exp $";
  *
  *	- traitement du Minitel 2
  *	- utilisation de syslog
+ *
+ *   Vincent Gillet
+ *
+ *	- dialogue avec client Window$ (option -w)
  */
 
 #define EXTERN
 
 #include <stdio.h>
+#include <ctype.h>
 #include "demon.h"
 #include "globald.h"
 
@@ -80,21 +85,38 @@ static int sock_service;
 
 #else
 
+#include <netinet/in.h>
 #define XTELD_INPUT	0
 #define XTELD_OUTPUT	1
 
 #endif /* NO_NETWORK */
 
+/* Tableau pour Emulateur sous Windaube */
+static char tablo_windaube[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  36,
+			 37, 102, 103,  36,  37, 102, 103,  60,
+			 61, 126, 127,  60,  61, 126, 127,  60,
+			 61, 126, 127,  60,  61, 126, 127,  60,
+			 61, 126, 127,  60,  61, 126, 127,  98,
+			 99,  98,  99, 102, 103, 102, 103, 122,
+			123, 122, 123, 126, 127, 126, 127, 122,
+			123, 122, 123, 126, 127, 126, 127, 122,
+			123, 122, 123, 126, 127, 126, 127, 102,
+			103, 102, 103, 102, 103, 102, 103, 126,
+			127, 126, 127, 126, 127, 126, 127, 126,
+			127, 126, 127, 126, 127, 126, 127, 126,
+			127, 126, 127, 126, 127, 126};
+
 static int fin_fils;
 static int pid_fils;
 static int nb_services;
 static fd_set a_lire;
 static struct timeval timeout;
 static time_t t_connexion;
-static char buf[256], service[256], utilisateur[256];
+static char buf[256], buf1[256], service[256], utilisateur[256];
 static char flag_connexion;
 static char parite;
-static Boolean flag_serveur_local;
+static Boolean flag_serveur_local, flag_crlf, flag_windaube;
 static int tuyau_in[2], tuyau_out[2];
 
 /* Syslog or not syslog ? */
@@ -119,6 +141,7 @@ void log_debug (fmt, p1, p2, p3, p4, p5, p6, p7)
 char *fmt;
 int  p1, p2, p3, p4, p5, p6, p7;
 {
+    fprintf (fp_console, "xteld[%d] ", getpid());
     fprintf (fp_console, fmt, p1, p2, p3, p4, p5, p6, p7);
     fprintf (fp_console, "\n\r");
 }
@@ -126,7 +149,7 @@ int  p1, p2, p3, p4, p5, p6, p7;
 
 void demande_fin_fils()
 {
-#ifdef DEBUG
+#ifdef DEBUG_XTELD
     log_debug ("Demande la fin du fils");
 #endif
     fin_fils = 1;
@@ -183,16 +206,14 @@ static void waitchild ()
 {
     int r;
 
-    while (wait (&r) > 0)
-	;
+    wait (&r);
 }
 
 void deconnexion ()
 {
-    int r;
     FILE *fplog;
 
-#ifdef DEBUG
+#ifdef DEBUG_XTELD
     log_debug ("Deconnecte !");
 #endif
 
@@ -206,7 +227,7 @@ void deconnexion ()
 	     * Envoi du code de connexion/fin puis de la sequence
 	     * de raccrochage
 	     */
-#ifdef DEBUG
+#ifdef DEBUG_XTELD
 	    log_debug ("Raccrochage Minitel");
 #endif
 	    write (fd_modem, "\x13I\x1b\x39\x67", 5);
@@ -247,17 +268,57 @@ static void test_mort_subite ()
     }
 }    
 
+/* Test chaine = nombre */
+static is_number (s)
+char *s;
+{
+    register char *p = s;
+
+    if (!s)
+	return 0;
+
+    while (*p) {
+	if (!isdigit(*p))
+	    return 0;
+
+	p++;
+    }
+
+    return 1;
+}
+
+/* Separation de chaine */
+static char *separe_chaine (s, c)
+char *s;
+char c;
+{
+    register char *p = s;
+    char *s1;
+    
+    while (*p) {
+	if (*p == c) {
+	    s1 = strdup (p+1);
+	    *p = 0;
+	    return s1;
+	}
+
+	p++;
+    }
+
+    return NULL;
+}
+
 /* 
  * Connexion a un service 
  */
 void appel_service (service_teletel)
 char *service_teletel;
 {
-    char c, *device_associe = NULL, *numero_direct;
+    char *device_associe = NULL, *numero_direct, *option = NULL;
     FILE *fplog;
     register int i;
 
-#ifdef DEBUG
+#ifdef DEBUG_XTELD
     log_debug ("connexion au service %s utilisateur %s", service_teletel, utilisateur);
 #endif
 
@@ -279,9 +340,13 @@ char *service_teletel;
     }
 
     /* Service local par "pipe" */
-    if (device_associe != NULL && strcmp (device_associe, "@pipe") == 0) {
+    if (device_associe != NULL && strncmp (device_associe, "@pipe", 5) == 0) {
 	flag_serveur_local = True;
 
+	/* Options du service local (pour l'instant "crlf") */
+	if (strcmp (&device_associe[5], ":crlf") == 0)
+	    flag_crlf = True;
+
 	if (pipe (tuyau_in) < 0) {
 #ifdef USE_SYSLOG
 	    log_err ("pipe: tuyau_in: %m");
@@ -301,7 +366,7 @@ char *service_teletel;
 	}
 
 	if (!fork()) { /* fiston */
-#ifdef DEBUG
+#ifdef DEBUG_XTELD
 	    log_debug ("le fiston = %d", getpid());
 #endif
 	    dup2 (tuyau_out[0], 0);
@@ -309,7 +374,14 @@ char *service_teletel;
 	    dup2 (tuyau_in[1], 1);
 	    close (tuyau_in[1]);
 
-	    if (execlp (service_teletel, service_teletel, NULL) < 0) {
+	    /* On recupere l'option eventuelle */
+	    option = separe_chaine (service_teletel, ' ');
+
+#ifdef DEBUG_XTELD
+	    log_debug ("service_teletel %s option -%s-", service_teletel, option);
+#endif
+
+	    if (execlp (service_teletel, service_teletel, option, NULL) < 0) {
 #ifdef USE_SYSLOG
 		log_err ("execlp: %m");
 #else
@@ -319,16 +391,30 @@ char *service_teletel;
 	    }
 	}
 	else {
-#ifdef DEBUG
+#ifdef DEBUG_XTELD
 	    log_debug ("le papa = %d", getpid());
 #endif
 	    fd_modem = tuyau_in[0];
 	}
     }
-    else { /* Connexion distante */
-	/* connexion */
+    else 
+#ifndef NO_NETWORK
+      /* Connexion TCP */
+      if (device_associe != NULL && strncmp (device_associe, "@tcp", 4) == 0) {
+	/* Recupere les parametres */
+	option = separe_chaine (service_teletel, ':');
+	if (is_number (option)) {
+	  fd_modem = c_clientbyport (service_teletel, atoi(option));
+	}
+	else {
+	  fd_modem = c_clientbyname (service_teletel, option);
+	}
+      }
+    else
+#endif
+      { /* Connexion MODEM */
 	if ((fd_modem = mydial(service_teletel, device_associe)) < 0) {
-#ifdef DEBUG
+#ifdef DEBUG_XTELD
 	    log_debug ("meurt (erreur)");
 #endif
 	    exit (1);
@@ -342,7 +428,7 @@ char *service_teletel;
     FD_ZERO (&a_lire);	
     FD_SET (fd_modem, &a_lire);
 	
-#ifdef DEBUG
+#ifdef DEBUG_XTELD
     log_debug ("Connecte !");
 #endif
 	
@@ -372,7 +458,7 @@ char *service_teletel;
 
 	    if (nread < 0 && errno == EINTR) {
 		/* select() interrompu par le SIGTERM du parent */
-#ifdef DEBUG
+#ifdef DEBUG_XTELD
 		log_debug ("select() interrompu par le SIGTERM du parent");
 #endif
 		continue;
@@ -380,7 +466,7 @@ char *service_teletel;
 	    
 	    if (nread == 0 || !FD_ISSET (fd_modem, &a_lire)) {
 		/* read = 0 (timeout) ==> deconnexion */
-#ifdef DEBUG
+#ifdef DEBUG_XTELD
 		log_debug ("read = 0 (timeout) ==> deconnexion");
 #endif
 		code_fin = 2;
@@ -390,7 +476,7 @@ char *service_teletel;
 	    size = read (fd_modem, buf, sizeof(buf));
 
 	    if (size <= 0) {
-#ifdef DEBUG
+#ifdef DEBUG_XTELD
 		log_debug ("size <= 0");
 #endif
 		code_fin = 3;
@@ -431,11 +517,27 @@ char *service_teletel;
 		}
 	    }
 
-	    write (XTELD_OUTPUT, buf, size); /* ecrit sur la connexion XTEL */
+	    /* Conversion LF --> CF/LF si serveur local ? */
+	    if (flag_serveur_local && flag_crlf) {
+		register int i, size1;
+
+		size1 = 0;
+		for (i = 0 ; i != size ; i++) {
+		    if (buf[i] == '\n') {
+			buf1[size1++] = '\n';
+			buf1[size1++] = '\r';
+		    }
+		    else
+			buf1[size1++] = buf[i];
+		}
+		write (XTELD_OUTPUT, buf1, size1);
+	    }
+	    else
+		write (XTELD_OUTPUT, buf, size);
 	}
 
 	/* Sortie du fils */
-#ifdef DEBUG
+#ifdef DEBUG_XTELD
 	log_debug ("meurt, code = %d", code_fin);
 #endif
 	exit (0);
@@ -446,7 +548,7 @@ char *service_teletel;
     /* creation du fichier de log */
     sprintf (buf, "/tmp/.xtel-%s", utilisateur);
     if ((fplog = fopen (buf, "w"))) {
-#ifdef DEBUG
+#ifdef DEBUG_XTELD
 	log_debug ("Creation du fichier de log %s", buf);
 #endif
 	chmod (buf, 0644);
@@ -505,6 +607,134 @@ err:
     return -1;
 }
 
+/* 
+ * Convertit la chaine en supprimant les caracteres accentues les
+ * plus courants.
+ */
+static void write_chaine_sans_accent (s)
+char *s;
+{
+    register unsigned char *p = s;
+    char c;
+
+    while (*p) {
+	if (*p >= 0xe0 && *p <= 0xe5)
+	    c = 'a';
+	if (*p >= 0xe8 && *p <= 0xeb )
+	   c = 'e';
+	else if (*p >= 0xec && *p <= 0xef)
+	    c = 'i';
+	else if (*p >= 0xf2 && *p <= 0xf6)
+	    c = 'o';
+	else if (*p >= 0xf9 && *p <= 0xf9)
+	    c = 'o';
+	else if (*p == 0xe7)
+	    c = 'c';
+	else
+	    c = *p;
+
+	p++;
+	write (XTELD_OUTPUT, &c, 1);
+    }
+}	
+
+static void trop_tard ()
+{
+#ifdef USE_SYSLOG
+    log_err ("Pas de reponse du client.\n");
+#else
+    fprintf (fp_console, "Pas de reponse du client.\n");
+#endif /* USE_SYSLOG */
+    exit (1);
+}
+
+/* La meme chose que "lire_chaine" appelee au lancement de xteld */
+int lire_initial(ch)
+char *ch;
+{
+    int ret,indice_service;
+    unsigned char l;
+    char *ptr;
+
+    /* Demande d'identification */
+    signal (SIGALRM, trop_tard);
+    alarm (20);
+    write (XTELD_OUTPUT, "\x1B\x39\x7b", 3);
+    read (XTELD_INPUT, service, 7);
+    signal (SIGALRM, SIG_IGN);
+
+    if ((!strncmp (service, "\x01SX1",4)) && service[6] == 4) {
+	/* SX1x ==> Emulateur Windows */
+	sprintf (buf, "\x1B\x39\x7A%c%c\x04", tablo_windaube[service[4]], tablo_windaube[service[5]]);
+	write (XTELD_OUTPUT, buf, 6);
+#ifdef DEBUG_XTELD
+	log_debug ("Emulateur Windows, repond : %s", service);
+	log_debug ("RAM = %s", buf);
+#endif
+	write (XTELD_OUTPUT, "\n\r Service : \r\n", 15);
+	/* Pour l'instant, on definit un seul utilisateur Window$: "windaube" */
+	strcpy (ch, "windaube");
+
+	for (indice_service=0; indice_service != nb_services; indice_service++) {
+	    if (service_autorise (indice_service)) {
+		sprintf (buf, "%2d - %s\r\n", indice_service, definition_services[indice_service].nom_service);
+		write_chaine_sans_accent (buf);
+	    }
+	}
+	write (XTELD_OUTPUT, "Choisissez un numero : ", 23);
+		
+	ptr = buf-1;
+	do {
+	    ptr++;
+	    ret = read (XTELD_INPUT, ptr, 1);
+#ifdef DEBUG_XTELD
+	    log_debug ("lu : %d", *ptr);
+#endif
+	    if (ret < 0 || ptr-buf > 5) goto err;
+	    write (XTELD_OUTPUT, ptr,1);
+	} while (*ptr != '\r');
+	*ptr = '\000';
+
+#ifdef DEBUG_XTELD
+	log_debug ("lu : %s",buf);
+#endif
+	if (atoi(buf) > nb_services || ! service_autorise (atoi(buf))) {
+	    sprintf (buf, "\r\n\r\n ** Choix interdit ! **\r\n");
+	    write (XTELD_OUTPUT, buf, strlen(buf));
+	    goto err;
+	}
+
+	/* Il semble que cela soit ncessaire... */
+	write (XTELD_OUTPUT, "\x1B[?31l\x0A\x1B\x3A\x32\x7E\x0A", 12);
+	write (XTELD_OUTPUT, "\x1B\x5B\x3F\x33\x32\x68", 6);
+	write (XTELD_OUTPUT, "\x1B\x5B\x3F\x33\x33\x68", 6);
+
+	/* nom du service */
+	strcpy (service, definition_services[atoi(buf)].nom_uucp);
+
+	return 1;
+    }
+
+    ch[l] = 0;
+
+    while (l) {
+	ret = read (XTELD_INPUT, ch, l);
+	if (ret < 0) goto err;
+	l -= ret;
+    }
+
+    return 0;
+
+err:
+#ifdef USE_SYSLOG
+    log_err ("read: %m");
+#else
+    fprintf (fp_console, "read: %s\n", sys_errlist[errno]);
+#endif /* USE_SYSLOG */
+    return -1;
+}
+
+
 /*
  *	Partie principale 
  */
@@ -512,16 +742,16 @@ main (ac, av)
 int ac;
 char **av;
 {
-    unsigned char c, l;
-    int ret;
+    unsigned char c;
     char fin_connexion = 0;
+    register int i;
 #ifdef NO_NETWORK
     int sock_ecoute, retry;
     static struct sockaddr_un unaddr;
-    register int i;
     struct stat statb;
+    int old_umask;
 #endif /* NO_NETWORK */
-    int indice_service = 0, nread;
+    int indice_service = 0;
 
 #ifdef USE_SYSLOG
     openlog("xteld", LOG_PID | LOG_CONS, LOG_DAEMON);
@@ -532,6 +762,19 @@ char **av;
     }
 #endif /* USE_SYSLOG */
 
+    i = 1;
+    while (i < ac) {
+	/* Option support Windaube */
+	if (strcmp (av[i], "-w") == 0) {
+#ifdef DEBUG_XTELD
+	    log_debug ("Option Windaube active !");
+#endif
+	    flag_windaube = True;
+	}
+
+	i++;
+    }
+
     if ((nb_lignes = lecture_configuration_lignes ()) <= 0) {
 	exit (1);
     }
@@ -596,6 +839,7 @@ char **av;
 	exit (1);
     }
 
+    old_umask = umask (0);
     unaddr.sun_family = AF_UNIX;
     strcpy (unaddr.sun_path, XTEL_UNIX_PATH);
 
@@ -619,17 +863,17 @@ char **av;
       exit (1);
     }
 
+    (void)umask (old_umask);
+
     /*
      * Attente de connexion...
      */
-
-
     for (;;) {
 
       /* Pour eviter les zombies */
       signal (SIGCHLD, waitchild);
 
-#ifdef DEBUG
+#ifdef DEBUG_XTELD
 	log_debug ("Attente de connexion...");
 #endif
 
@@ -655,25 +899,28 @@ char **av;
 
 #endif /* NO_NETWORK */
 
-#ifdef DEBUG
+#ifdef DEBUG_XTELD
 	    log_debug ("Connexion XTEL...");
 #endif
 
 	    /* Lecture du nom d'utilisateur */
-
-	    if (lire_chaine(utilisateur) < 0)
-		goto fin_xteld;
-
-#ifdef DEBUG
-	    log_debug ("utilisateur = %s", utilisateur);
-#endif
+	    if (flag_windaube) {
+		if (lire_initial(utilisateur) < 0)
+		    goto fin_xteld;
+		write (XTELD_OUTPUT, "\x1B\x54\x20\x0F Appel...", 13);
+		appel_service (service);
+	    }
+	    else {
+		if (lire_chaine(utilisateur) < 0)
+		    goto fin_xteld;
+	    }
 
 	    /* lecture connexion XTEL */
 	    while (!fin_connexion && read (XTELD_INPUT, &c, 1) == 1) {
 
 		if (flag_connexion) {
 		    if (c == VALEUR_COMMANDE_FIN) {
-#ifdef DEBUG
+#ifdef DEBUG_XTELD
 			log_debug ("tue le fils");
 #endif		      
 			kill (pid_fils, SIGTERM);
@@ -722,7 +969,7 @@ char **av;
 				c = strlen (definition_services[indice_service].nom_service);
 				write (XTELD_OUTPUT, &c, 1);
 				write (XTELD_OUTPUT, definition_services[indice_service].nom_service, c);
-#ifdef DEBUG
+#ifdef DEBUG_XTELD
 				log_debug ("service: %s", definition_services[indice_service].nom_service);
 #endif		      
 			    }
@@ -740,7 +987,7 @@ char **av;
 			c = strlen (definition_services[indice_service].nom_uucp);
 			write (XTELD_OUTPUT, &c, 1);
 			write (XTELD_OUTPUT, definition_services[indice_service].nom_uucp, c);
-#ifdef DEBUG
+#ifdef DEBUG_XTELD
 			log_debug ("%s", definition_services[indice_service].nom_uucp);
 #endif		      
 			indice_service++;
@@ -760,7 +1007,7 @@ fin_xteld:
 	    /* Si xtel a ete tue sauvagement, tue le fils */
 	    test_mort_subite ();
 
-#ifdef DEBUG
+#ifdef DEBUG_XTELD
 	    log_debug ("Fin du service");
 #endif
 	    /* Fin du service */
@@ -768,7 +1015,7 @@ fin_xteld:
 
 	} /* if !fork */
 
-#ifdef DEBUG
+#ifdef DEBUG_XTELD
 	log_debug ("Ferme la socket de service");
 #endif
 	close (sock_service);
@@ -776,8 +1023,8 @@ fin_xteld:
 
 #else
 
-#ifdef DEBUG
-    log_debug ("%d meurt (OK), errno = %d", getpid(), errno);
+#ifdef DEBUG_XTELD
+    log_debug ("%d meurt (OK), errno = %d ICI", getpid(), errno);
 #endif
 
     /* Si xtel a ete tue sauvagement, tue le fils */
